@@ -1,24 +1,51 @@
 # -*- coding: utf-8 -*-
-"""Functions from market data"""
+"""
+Functions to create strategies.
 
-__author__ = "Miguel Martin"
-__version__ = "1"
+All of the functions return a *programatic function* which calculates the given function for some OCHL data.
+
+To build a function not included in the package, use the following template::
+
+    def function_name(params, target='close'):
+        def return_function(data):
+            your code goes here
+        return return_function
+
+"""
 
 import numpy as np
 
 
 def days_to_constant(days, order=1):
+    #TODO: Finish documenting
+    """
+    .. _days_to_constant:
+
+    Calculates the constant for exponential smoothing from a given window size
+
+    :param days: Window size
+    :type days: int
+    :param order: [description], defaults to 1
+    :type order: int, optional
+    """
     return float("%.3f" % (1 - pow(1 - 2 / (days + 1), 1 / order)))
 
 
 def get_column(column):
     def return_function(data):
-        return data[column]
+        return data[column].copy()
 
     return return_function
 
+def trailing(target="close", days=40, over_under="under"):
+    """
+    Calculates the if the target is trailing under or over the current in the past days.
 
-def trailling(target="close", days=40, over_under="under"):
+    :param target: Data column to use for the calculation, defaults to "close"
+    :type target: str
+    :param days: Size of the window in days, defaults to 40
+    :type days: int
+    """
     def f():
         def g(x):
             return all(x[-1] > x[:-1])
@@ -26,7 +53,7 @@ def trailling(target="close", days=40, over_under="under"):
         return g
 
     def return_function(data):
-        column_name = "trailling_" + over_under + "_" + str(days) + "_of_" + target
+        column_name = f"trailling_{over_under}_{days}_of_{target}"
         if column_name not in data.columns:
             data[column_name] = (
                 data[target].rolling(window=days, min_periods=1).apply(f(), raw=True)
@@ -37,8 +64,15 @@ def trailling(target="close", days=40, over_under="under"):
 
 
 def rolling_std(days, target="close"):
+    """Calculates the rolling standard deviation
+
+    :param days: Size of the window in days
+    :type days: int
+    :param target: Data column to use for the calculation, defaults to "close"
+    :type target: str, optional
+    """
     def return_function(data):
-        column_name = "rolling_std_" + str(days) + "_of_" + target
+        column_name = f"rolling_std_{days}_of_{target}"
         if column_name not in data.columns:
             data[column_name] = data[target].rolling(days, min_periods=2).std()
         return data[column_name].copy()
@@ -47,8 +81,15 @@ def rolling_std(days, target="close"):
 
 
 def moving_average(days, target="close"):
+    """Calculates the rolling moving average
+
+    :param days: Size of the window in days
+    :type days: int
+    :param target: Data column to use for the calculation, defaults to "close"
+    :type target: str, optional
+    """
     def return_function(data):
-        column_name = "moving_average_" + str(days) + "_of_" + target
+        column_name = f"moving_average_{days}_of_{target}"
         if column_name not in data.columns:
             data[column_name] = data[target].rolling(days, min_periods=1).mean()
         return data[column_name].copy()
@@ -57,6 +98,13 @@ def moving_average(days, target="close"):
 
 
 def weighted_moving_average(days, weights):
+    """Calculates the rolling weighted moving average. The weights list should have the size of the days window.
+
+    :param days: Size of the window
+    :type days: int
+    :param weights: Weights to use for the calculation
+    :type weights: list(int)
+    """
     def f(w):
         def g(x):
             return (w * x).sum() / sum(w)
@@ -65,7 +113,7 @@ def weighted_moving_average(days, weights):
 
     def return_function(data):
         if len(weights) == days:
-            column_name = "weighted_moving_average_" + str(days)
+            column_name = f"weighted_moving_average_{days}"
             if column_name not in data.columns:
                 data[column_name] = (
                     data["close"].rolling(window=days).apply(f(weights), raw=True)
@@ -78,6 +126,15 @@ def weighted_moving_average(days, weights):
 
 
 def step_weighting_ma(days, first_weight=1, step=1):
+    """Calculates the weighted moving average using stepped weights
+
+    :param days: Window size
+    :type days: int
+    :param first_weight: Weight of the first data point, defaults to 1
+    :type first_weight: int, optional
+    :param step: Weight increment for each data point, defaults to 1
+    :type step: int, optional
+    """
     def f(w):
         def g(x):
             return (w * x).sum() / sum(w)
@@ -86,7 +143,7 @@ def step_weighting_ma(days, first_weight=1, step=1):
 
     def return_function(data):
         weights = np.linspace(first_weight, (days * step) + first_weight - step, days)
-        column_name = "step_weighting_ma_" + str(days)
+        column_name = f"step_weighting_ma_{days}"
         if column_name not in data.columns:
             data[column_name] = (
                 data["close"].rolling(window=days).apply(f(weights), raw=True)
@@ -97,6 +154,15 @@ def step_weighting_ma(days, first_weight=1, step=1):
 
 
 def percentage_weighting_ma(days, last_weight=1, step=0.5):
+    """Calculates the weighted moving average using backwards set weights. F.e. for a given last days=3, weight=1, step=0.5, the wieghts would be [0.25, 0.5, 1]
+
+    :param days: Window size
+    :type days: int
+    :param last_weight: Weights of the last data point, defaults to 1
+    :type last_weight: int, optional
+    :param step: Weight multiplier for each previous data point, defaults to 0.5
+    :type step: float, optional
+    """
     def f(w):
         def g(x):
             return (w * x).sum() / sum(w)
@@ -108,7 +174,7 @@ def percentage_weighting_ma(days, last_weight=1, step=0.5):
         weights[-1] = last_weight
         for i in range(1, days):
             weights[-1 * (i + 1)] = weights[-1 * i] * step
-        column_name = "percentage_weighting_ma_" + str(days)
+        column_name = f"percentage_weighting_ma_{days}"
         if column_name not in data.columns:
             data[column_name] = (
                 data["close"].rolling(window=days).apply(f(weights), raw=True)
@@ -119,6 +185,13 @@ def percentage_weighting_ma(days, last_weight=1, step=0.5):
 
 
 def triangular_weighting_ma(days=10, shape="linear"):
+    """Calculates the weighted moving average with the weights having a triangular shape
+
+    :param days: Window size, defaults to 10
+    :type days: int, optional
+    :param shape: Type of shape. Can be "linear" or "gaussian" , defaults to "linear"
+    :type shape: str, optional
+    """
     def f(w):
         def g(x):
             return (w * x).sum() / sum(w)
@@ -135,10 +208,10 @@ def triangular_weighting_ma(days=10, shape="linear"):
         return weights
 
     def gaussian_triangular_weights():
-        return 0
+        return NotImplementedError()
 
     if shape not in ["linear", "gaussian"]:
-        raise Exception(type, " is not a type of triangular moving average")
+        raise NotImplementedError()
     else:
         weight_options = {
             "linear": linear_triangular_weights,
@@ -148,7 +221,7 @@ def triangular_weighting_ma(days=10, shape="linear"):
         weights_values = weights_func()
 
     def return_function(data):
-        column_name = str(shape) + "_trianguar_weighting_ma_" + str(days)
+        column_name = f"{shape}_trianguar_weighting_ma_{days}"
         if column_name not in data.columns:
             data[column_name] = (
                 data["close"].rolling(window=days).apply(f(weights_values), raw=True)
@@ -159,6 +232,8 @@ def triangular_weighting_ma(days=10, shape="linear"):
 
 
 def pivot_point_weighting_ma(days):
+    #TODO: Review the implementation of this function. Pivot point in the internet is referred by the internet to the moving average
+    #of (close+high+low)/3. Refer to the book.
     def f(w):
         def g(x):
             return (w * x).sum() / sum(w)
@@ -172,7 +247,7 @@ def pivot_point_weighting_ma(days):
 
     def return_function(data):
         weights = pivot_point_weights()
-        column_name = "pivot_point_weighting_ma_" + str(days)
+        column_name = f"pivot_point_weighting_ma_{days}"
         if column_name not in data.columns:
             data[column_name] = (
                 data["close"].rolling(window=days).apply(f(weights), raw=True)
@@ -182,12 +257,19 @@ def pivot_point_weighting_ma(days):
     return return_function
 
 
-def geometric_moving_average(days):
+def geometric_moving_average(days, target='close'):
+    """Calculates the geometric moving average
+
+    :param days: Window size
+    :type days: int
+    :param target: Data column to use, defaults to "close"
+    :type target: str, optional
+    """
     def return_function(data):
-        column_name = "geometric_moving_average_" + str(days)
+        column_name = f"geometric_moving_average_{days}"
         if column_name not in data.columns:
             data[column_name] = pow(
-                data["close"].rolling(window=days).apply(np.prod, raw=True), 1 / days
+                data[target].rolling(window=days).apply(np.prod, raw=True), 1 / days
             )
         return data[column_name].copy()
 
@@ -195,24 +277,35 @@ def geometric_moving_average(days):
 
 
 def exponential_smoothing(constant, target="close"):
+    """Calculates the exponential smoothing
+
+    :param constant: Smoothing constant for the calculation. Use function days_to_constant_
+    :type constant: float
+    :param target: Data column to use, defaults to "close"
+    :type target: str, optional
+    """
     def return_function(data):
-        column_name = "exponential_smoothing_" + str(constant) + "%_of_" + target
+        column_name = f"exponential_smoothing_{constant}%_of_{target}"
         if column_name not in data.columns:
             data[column_name] = data[target].copy()
             for i in data[data[target].notnull()].index[1:]:
-                # print(data[column_name].loc[:i])
                 data.loc[i, column_name] = data[column_name].loc[:i][-2] + constant * (
-                    data[target].loc[i] - data[column_name].loc[:i][-2]
-                )
+                    data[target].loc[i] - data[column_name].loc[:i][-2])
+
         return data[column_name].copy()
 
     return return_function
 
 
 def second_order_exponential_smoothing(constant):
+    """Calculates the second order exponential smoothing of close
+
+    :param constant: Smoothing constant for the calculation. Use function days_to_constant_
+    :type constant: float
+    """
     def return_function(data):
         first_order = exponential_smoothing(constant)(data)
-        column_name = "second_order_exponential_smoothing_" + str(constant) + "%"
+        column_name = f"second_order_exponential_smoothing_{constant}%"
         if column_name not in data.columns:
             data[column_name] = first_order.copy()
             for i in range(1, len(data["close"])):
@@ -225,9 +318,14 @@ def second_order_exponential_smoothing(constant):
 
 
 def third_order_exponential_smoothing(constant):
+    """Calculates the second order exponential smoothing of close
+
+    :param constant: Smoothing constant for the calculation. Use function days_to_constant_
+    :type constant: float
+    """
     def return_function(data):
         second_order = second_order_exponential_smoothing(constant)(data)
-        column_name = "third_order_exponential_smoothing_" + str(constant) + "%"
+        column_name = f"third_order_exponential_smoothing_{constant}%"
         if column_name not in data.columns:
             data[column_name] = second_order.copy()
             for i in range(1, len(data["close"])):
@@ -240,10 +338,9 @@ def third_order_exponential_smoothing(constant):
 
 
 def lag_correction_exponential_smoothing(constant):
+    #TODO: View the book for explanation
     def return_function(data):
-        column_name = (
-            "lag_correction_exponential_smoothing_" + str(constant * 100) + "%"
-        )
+        column_name = (f"lag_correction_exponential_smoothing_{constant*100}%")
         if column_name not in data.columns:
             first_order = exponential_smoothing(constant)(data)
             data[column_name] = data["close"] - first_order
@@ -258,8 +355,13 @@ def lag_correction_exponential_smoothing(constant):
 
 
 def double_moving_average(days):
+    """Calculates the double moving average of close
+
+    :param days: Window size
+    :type days: int
+    """
     def return_function(data):
-        column_name = "double_moving_average_" + str(days)
+        column_name = f"double_moving_average_{days}"
         if column_name not in data.columns:
             simple_ma = moving_average(days)(data)
             data[column_name] = simple_ma.rolling(days, min_periods=1).mean()
@@ -269,8 +371,15 @@ def double_moving_average(days):
 
 
 def double_smoothed_momentum(days1, days2):
+    """Calculates the momentum, smoothed by two consecutive exponential smoothings
+    
+    :param days1: First smoothing window size
+    :type days1: int
+    :param days2: Second smoothing window size
+    :type days2: int
+    """
     def return_function(data):
-        column_name = "DSM" + str(days1) + "/" + str(days2)
+        column_name = f"DSM{days1}/{days2}"
         if column_name not in data.columns:
             momentum_val = momentum(days=days1)(data)
             constant = days_to_constant(days1)
@@ -287,8 +396,9 @@ def double_smoothed_momentum(days1, days2):
 
 
 def regularized_exponential_ma(days, weight, target="close"):
+    #TODO: Refer to book
     def return_function(data):
-        column_name = "REMA_" + str(days) + "/" + str(weight) + "_of_" + str(target)
+        column_name = f"REMA_{days}/{weight}_of_{target}"
         if column_name not in data.columns:
             c = days_to_constant(days)
             data[column_name] = data[target].copy()
@@ -304,8 +414,15 @@ def regularized_exponential_ma(days, weight, target="close"):
 
 
 def hull_moving_average(period=16, target="close"):
+    """Calculates the hull moving average
+
+    :param period: Window size, defaults to 16
+    :type period: int, optional
+    :param target: Data colum to use, defaults to "close"
+    :type target: str, optional
+    """
     def return_function(data):
-        column_name = "hull_moving_average_" + str(period) + "_of_" + str(target)
+        column_name = f"hull_moving_average_{period}_of_{target}"
         if column_name not in data.columns:
             period_ma = moving_average(period)(data)
             period2_ma = moving_average(int(period / 2))(data)
@@ -320,14 +437,19 @@ def hull_moving_average(period=16, target="close"):
 
 
 def upper_keltner_channel(days=10):
+    """Calculates the upper keltner channel
+
+    :param days: Window size, defaults to 10
+    :type days: int, optional
+    """
     def return_function(data):
-        column_name = "upper_keltner_" + str(days)
+        column_name = f"upper_keltner_{days}"
         if column_name not in data.columns:
             average_daily_price = (data["close"] + data["high"] + data["low"]) / 3
-            if ("moving_average_" + str(days) + "_of_close") not in data.columns:
+            if (f"moving_average_{days}_of_close") not in data.columns:
                 moving_average(days)(data)
             data[column_name] = (
-                data["moving_average_" + str(days) + "_of_close"] + average_daily_price
+                data[f"moving_average_{days}_of_close"] + average_daily_price
             )
         return data[column_name].copy()
 
@@ -335,14 +457,19 @@ def upper_keltner_channel(days=10):
 
 
 def lower_keltner_channel(days=10):
+    """Calculates the lower keltner channel
+
+    :param days: Window size, defaults to 10
+    :type days: int, optional
+    """
     def return_function(data):
-        column_name = "lower_keltner_" + str(days)
+        column_name = f"lower_keltner_{days}"
         if column_name not in data.columns:
             average_daily_price = (data["close"] + data["high"] + data["low"]) / 3
-            if ("moving_average_" + str(days) + "_of_close") not in data.columns:
+            if (f"moving_average_{days}_of_close") not in data.columns:
                 moving_average(days)(data)
             data[column_name] = (
-                data["moving_average_" + str(days) + "_of_close"] - average_daily_price
+                data[f"moving_average_{days}_of_close"] - average_daily_price
             )
         return data[column_name].copy()
 
@@ -350,15 +477,17 @@ def lower_keltner_channel(days=10):
 
 
 def upper_percentage_band(c, band_target, center_target):
+    """Calculates the upper percentage band
+
+    :param c: multiplier constant
+    :type c: float
+    :param band_target: Data column used for the band displacement
+    :type band_target: str
+    :param center_target: Data column used for the center of the channel
+    :type center_target: str
+    """
     def return_function(data):
-        column_name = (
-            "upper_percentage_band_"
-            + str(c)
-            + "_of_"
-            + band_target.name
-            + "_over_"
-            + center_target.name
-        )
+        column_name = f"upper_percentage_band_{c}_of_{band_target.name}_over_{center_target.name}"
         if column_name not in data.columns:
             print(center_target)
             data[column_name] = c * band_target + center_target.values
@@ -368,15 +497,17 @@ def upper_percentage_band(c, band_target, center_target):
 
 
 def lower_percentage_band(c, band_target, center_target):
+    """Calculates the lower percentage band
+
+    :param c: multiplier constant
+    :type c: float
+    :param band_target: Data column used for the band displacement
+    :type band_target: str
+    :param center_target: Data column used for the center of the channel
+    :type center_target: str
+    """
     def return_function(data):
-        column_name = (
-            "lower_percentage_band_"
-            + str(c)
-            + "_of_"
-            + band_target.name
-            + "_under_"
-            + center_target.name
-        )
+        column_name = f"lower_percentage_band_{c}_of_{band_target.name}_under_{center_target.name}"
         if column_name not in data.columns:
             data[column_name] = -c * band_target + center_target.values
         return data[column_name].copy()
@@ -385,8 +516,15 @@ def lower_percentage_band(c, band_target, center_target):
 
 
 def upper_absolute_band(value, target):
+    """Calculates the upper absolute band
+
+    :param value: Constant for band displacement
+    :type value: float
+    :param target: Data column for the center of the band
+    :type target: str
+    """
     def return_function(data):
-        column_name = "upper_absolute_band_" + str(value) + "_over_" + target.name
+        column_name = f"upper_absolute_band_{value}_over_{target.name}"
         if column_name not in data.columns:
             data[column_name] = value + target
         return data[column_name].copy()
@@ -395,8 +533,15 @@ def upper_absolute_band(value, target):
 
 
 def lower_absolute_band(value, target):
+    """Calculates the lower absolute band
+
+    :param value: Constant for band displacement
+    :type value: float
+    :param target: Data column for the center of the band
+    :type target: str
+    """
     def return_function(data):
-        column_name = "lower_absolute_band_" + str(value) + "_under_" + target.name
+        column_name = f"lower_absolute_band_{value}_under_{target.name}"
         if column_name not in data.columns:
             data[column_name] = -value + target
         return data[column_name].copy()
@@ -405,8 +550,17 @@ def lower_absolute_band(value, target):
 
 
 def upper_bollinger_band(mean_days, std_days, c):
+    """Calculates the upper bollinger band
+
+    :param mean_days: Window size for the band center
+    :type mean_days: int
+    :param std_days: Window size for the band displacement
+    :type std_days: str
+    :param c: Multiplier constant
+    :type c: float
+    """
     def return_function(data):
-        column_name = "upper_bollinger_band_" + str(mean_days) + "/" + str(std_days)
+        column_name = f"upper_bollinger_band_{mean_days}/{std_days}"
         if column_name not in data.columns:
             std_dev = rolling_std(std_days)(data)
             ma = moving_average(mean_days)(data)
@@ -417,8 +571,17 @@ def upper_bollinger_band(mean_days, std_days, c):
 
 
 def lower_bollinger_band(mean_days, std_days, c):
+    """Calculates the lower bollinger band
+
+    :param mean_days: Window size for the band center
+    :type mean_days: int
+    :param std_days: Window size for the band displacement
+    :type std_days: str
+    :param c: Multiplier constant
+    :type c: float
+    """
     def return_function(data):
-        column_name = "lower_bollinger_band_" + str(mean_days) + "/" + str(std_days)
+        column_name = f"lower_bollinger_band_{mean_days}/{std_days}"
         if column_name not in data.columns:
             std_dev = rolling_std(std_days)(data)
             ma = moving_average(mean_days)(data)
@@ -429,40 +592,24 @@ def lower_bollinger_band(mean_days, std_days, c):
 
 
 def upper_volatility_band(c, dev_target, band_target, center_target):
+    """Calculates the upper volatility band
+
+    :param c: Multiplier constant
+    :type c: float
+    :param dev_target: Used for band displacement. Can be a constant or a function
+    :type dev_target: function or float
+    :param band_target: Used for band displacement. Can be a constant or a function
+    :type band_target: function or float
+    :param center_target: Data column for the band center
+    :type center_target: str
+    """
     def return_function(data):
         if hasattr(band_target, "name") & hasattr(dev_target, "name"):
-            column_name = (
-                "upper_volatility_band_"
-                + str(c)
-                + "_times_"
-                + band_target.name
-                + "&"
-                + dev_target.name
-                + "_over_"
-                + center_target.name
-            )
+            column_name = f"upper_volatility_band_{c}_times_{band_target.name}&{dev_target.name}_over_{center_target.name}"
         elif hasattr(band_target, "name"):
-            column_name = (
-                "upper_volatility_band_"
-                + str(c)
-                + "_times_"
-                + band_target.name
-                + "&"
-                + str(dev_target)
-                + "_over_"
-                + center_target.name
-            )
+            column_name = f"upper_volatility_band_{c}_times_{band_target.name}&{dev_target}_over_{center_target.name}"
         else:
-            column_name = (
-                "upper_volatility_band_"
-                + str(c)
-                + "_times_"
-                + str(band_target)
-                + "&"
-                + str(dev_target)
-                + "_over_"
-                + center_target.name
-            )
+            column_name = f"upper_volatility_band_{c}_times_{band_target}&{dev_target}_over_{center_target.name}"
         if column_name not in data.columns:
             data[column_name] = center_target + c * dev_target * band_target
         return data[column_name].copy()
@@ -471,40 +618,24 @@ def upper_volatility_band(c, dev_target, band_target, center_target):
 
 
 def lower_volatility_band(c, dev_target, band_target, center_target):
+    """Calculates the lower volatility band
+
+    :param c: Multiplier constant
+    :type c: float
+    :param dev_target: Used for band displacement. Can be a constant or a function
+    :type dev_target: function or float
+    :param band_target: Used for band displacement. Can be a constant or a function
+    :type band_target: function or float
+    :param center_target: Data column for the band center
+    :type center_target: str
+    """
     def return_function(data):
         if hasattr(band_target, "name") & hasattr(dev_target, "name"):
-            column_name = (
-                "lower_volatility_band_"
-                + str(c)
-                + "_times_"
-                + band_target.name
-                + "&"
-                + dev_target.name
-                + "_under_"
-                + center_target.name
-            )
+            column_name = f"lower_volatility_band_{c}_times_{band_target.name}&{dev_target.name}_under_{center_target.name}"
         elif hasattr(band_target, "name"):
-            column_name = (
-                "lower_volatility_band_"
-                + str(c)
-                + "_times_"
-                + band_target.name
-                + "&"
-                + str(dev_target)
-                + "_under_"
-                + center_target.name
-            )
+            column_name = f"lower_volatility_band_{c}_times_{band_target.name}&{dev_target}_under_{center_target.name}"
         else:
-            column_name = (
-                "lower_volatility_band_"
-                + str(c)
-                + "_times_"
-                + str(band_target)
-                + "&"
-                + str(dev_target)
-                + "_under_"
-                + center_target.name
-            )
+            column_name = f"lower_volatility_band_{c}_times_{band_target}&{dev_target}_under_{center_target.name}"
         if column_name not in data.columns:
             data[column_name] = center_target - c * dev_target * band_target
         return data[column_name].copy()
@@ -513,8 +644,15 @@ def lower_volatility_band(c, dev_target, band_target, center_target):
 
 
 def momentum(days=1, target="close"):
+    """Calculates the momentum
+
+    :param days: Window size, defaults to 1
+    :type days: int, optional
+    :param target: Data column to use, defaults to "close"
+    :type target: str, optional
+    """
     def return_function(data):
-        column_name = "momentum" + str(days) + "_of_" + str(target)
+        column_name = f"momentum_{days}_of_{target}"
         if column_name not in data.columns:
             data[column_name] = data[target].diff(periods=days)
         return data[column_name].copy()
@@ -523,8 +661,15 @@ def momentum(days=1, target="close"):
 
 
 def momentum_percentage(days=1, target="close"):
+    """Calculates the momentum as a percentage
+
+    :param days: Window size, defaults to 1
+    :type days: int, optional
+    :param target: Data colum to use, defaults to "close"
+    :type target: str, optional
+    """
     def return_function(data):
-        column_name = "momentum_percentage" + str(days) + "_of_" + str(target)
+        column_name = f"momentum_percentage_{days}_of_{target}"
         if column_name not in data.columns:
             data[column_name] = data[target].diff(periods=days)
             data.loc[days:, column_name] = data[column_name][days:].divide(
@@ -535,11 +680,16 @@ def momentum_percentage(days=1, target="close"):
     return return_function
 
 
-def MACD_line(slow_trend, fast_trend, signal):
+def MACD_line(slow_trend=26, fast_trend=12):
+    """Calculates the MACD line
+
+    :param slow_trend: Slow window size, defaults to 26
+    :type slow_trend: int, optional
+    :param fast_trend: Fast window size, defaults to 12
+    :type fast_trend: int, optional
+    """
     def return_function(data):
-        column_name = (
-            "MACD_line_" + str(slow_trend) + "/" + str(fast_trend) + "/" + str(signal)
-        )
+        column_name = f"MACD_line_{slow_trend}/{fast_trend}"
         if column_name not in data.columns:
             slow_trend_data = exponential_smoothing(days_to_constant(slow_trend))(data)
             fast_trend_data = exponential_smoothing(days_to_constant(fast_trend))(data)
@@ -549,11 +699,18 @@ def MACD_line(slow_trend, fast_trend, signal):
     return return_function
 
 
-def MACD_signal(slow_trend, fast_trend, signal):
+def MACD_signal(signal, slow_trend=26, fast_trend=12):
+    """Calculates the MACD signal
+
+    :param signal: Exponential smoothing window size of the MACD line
+    :type signal: int
+    :param slow_trend: Slow window size of MACD line, defaults to 26
+    :type slow_trend: int, optional
+    :param fast_trend: Fast windown size of MACD line, defaults to 12
+    :type fast_trend: int, optional
+    """
     def return_function(data):
-        column_name = (
-            "MACD_signal_" + str(slow_trend) + "/" + str(fast_trend) + "/" + str(signal)
-        )
+        column_name = f"MACD_signal_{slow_trend}/{fast_trend}/{signal}"
         if column_name not in data.columns:
             macd_line = MACD_line(slow_trend, fast_trend, signal)(data)
             data[column_name] = exponential_smoothing(
@@ -564,19 +721,21 @@ def MACD_signal(slow_trend, fast_trend, signal):
     return return_function
 
 
-def MACD_histogram(slow_trend, fast_trend, signal):
+def MACD_histogram(signal, slow_trend=26, fast_trend=12):
+    """Calculates the MACD histogram
+
+    :param signal: Exponential smoothing window size of the MACD line
+    :type signal: int
+    :param slow_trend: Slow window size of MACD line, defaults to 26
+    :type slow_trend: int, optional
+    :param fast_trend: Fast windown size of MACD line, defaults to 12
+    :type fast_trend: int, optional
+    """
     def return_function(data):
-        column_name = (
-            "MACD_histogram_"
-            + str(slow_trend)
-            + "/"
-            + str(fast_trend)
-            + "/"
-            + str(signal)
-        )
+        column_name = f"MACD_histogram_{slow_trend}/{fast_trend}/{signal}"
         if column_name not in data.columns:
-            macd_line = MACD_line(slow_trend, fast_trend, signal)(data)
-            macd_signal = MACD_signal(slow_trend, fast_trend, signal)(data)
+            macd_line = MACD_line(slow_trend, fast_trend)(data)
+            macd_signal = MACD_signal(signal, slow_trend, fast_trend)(data)
             data[column_name] = macd_line - macd_signal
         return data[column_name].copy()
 
@@ -584,8 +743,15 @@ def MACD_histogram(slow_trend, fast_trend, signal):
 
 
 def divergence_index(slow_trend, fast_trend):
+    """Calculates the divergence index
+
+    :param slow_trend: Slow window size
+    :type slow_trend: int
+    :param fast_trend: Fast window size
+    :type fast_trend: int
+    """
     def return_function(data):
-        column_name = "divergence_index" + str(slow_trend) + "/" + str(fast_trend)
+        column_name = f"divergence_index_{slow_trend}/{fast_trend}"
         if column_name not in data.columns:
             fast_trend_data = moving_average(fast_trend)(data)
             slow_trend_data = moving_average(slow_trend)(data)
@@ -599,10 +765,17 @@ def divergence_index(slow_trend, fast_trend):
 
 
 def average_true_range(days=10, target="close"):
+    """Calculates the average true range
+
+    :param days: Window size, defaults to 10
+    :type days: int, optional
+    :param target: Data column to use, defaults to "close"
+    :type target: str, optional
+    """
     def return_function(data):
-        column_name = "average_true_range_" + str(days) + "_of_" + target
-        high_col_name = "high" + target[5:]
-        low_col_name = "low" + target[5:]
+        column_name = f"average_true_range_{days}_of_{target}"
+        high_col_name = f"high{target[5:]}"
+        low_col_name = f"low{target[5:]}"
         if column_name not in data.columns:
             true_range = data[high_col_name] - data[low_col_name]
             data[column_name] = true_range.rolling(days, min_periods=1).mean()
@@ -612,13 +785,18 @@ def average_true_range(days=10, target="close"):
 
 
 def stochastic_percentageK(days):
+    """Calculates the stochastic K
+
+    :param days: Window size
+    :type days: int
+    """
     def f(x):
         lowest = x["low"].nsmallest(1)
         highest = x["high"].max()
         return 100 * (x["close"].iloc[-1] - lowest) / (highest - lowest)
 
     def return_function(data):
-        column_name = "stochastic_percentageK_" + str(days)
+        column_name = f"stochastic_percentageK_{days}"
         if column_name not in data.columns:
             data[column_name] = np.zeros(len(data))
             column_index = data.columns.get_loc(column_name)
@@ -633,9 +811,16 @@ def stochastic_percentageK(days):
 
 
 def stochastic_percentageD(days, daysK):
+    """Calculates the stochastic D
+
+    :param days: Window size
+    :type days: int
+    :param daysK: Window size for the stochastic K
+    :type daysK: int
+    """
     def return_function(data):
-        column_name_K = "stochastic_percentageK_" + str(daysK)
-        column_name = "stochastic_percentageD_" + str(days) + "/" + str(daysK)
+        column_name_K = f"stochastic_percentageK_{days}"
+        column_name = f"stochastic_percentageD_{days}/{daysK}"
         if column_name not in data.columns:
             stochastic_percentageK(daysK)(data)
             data[column_name] = data[column_name_K].rolling(days).mean()
@@ -645,6 +830,7 @@ def stochastic_percentageD(days, daysK):
 
 
 def ADoscillator():
+    """Calculates the A/D oscillator"""
     def f(x):
         buying_power = x["high"] - x["open"]
         selling_power = x["close"] - x["low"]
@@ -661,6 +847,11 @@ def ADoscillator():
 
 
 def percentageR(days):
+    """Calculates the stochastic R
+    
+    :param days: Window size
+    :type days: int
+    """
     def f(x):
         highest = x["high"].max()
         lowest = x["low"].min()
@@ -668,7 +859,7 @@ def percentageR(days):
         return buying_power / (highest - lowest)
 
     def return_function(data):
-        column_name = "percentageR_" + str(days)
+        column_name = f"percentageR_{days}"
         if column_name not in data.columns:
             data[column_name] = np.zeros(len(data))
             column_index = data.columns.get_loc(column_name)
@@ -681,6 +872,11 @@ def percentageR(days):
 
 
 def RSI(days):
+    """Calculates the RSI
+
+    :param days: Window size
+    :type days: int
+    """
     def f():
         def g(x):
             AU = x[x > 0].sum()
@@ -694,7 +890,7 @@ def RSI(days):
         return g
 
     def return_function(data):
-        column_name = "RSI_" + str(days)
+        column_name = f"RSI_{days}"
         if column_name not in data.columns:
             mom_name = momentum()(data).name
             data[column_name] = (
@@ -706,6 +902,7 @@ def RSI(days):
 
 
 def special_k():
+    """Calculates the special k indicator"""
     def return_function(data):
         column_name = "special_k"
         if column_name not in data.columns:
